package org.javabuilders.swing.mydoggy;

import java.awt.Dimension;

import org.javabuilders.BuilderConfig;
import org.javabuilders.handler.type.DimensionAsValueHandler;
import org.javabuilders.swing.SwingBuilder;
import org.javabuilders.swing.mydoggy.handler.type.MyDoggyToolWindowManagerTypeHandler;
import org.noos.xing.mydoggy.plaf.MyDoggyContent;
import org.noos.xing.mydoggy.plaf.MyDoggyToolWindowManager;

/**
 * Registers the SwingX components into the specified Java Builder (by default the Java Swing Builder of course)
 * 
 * @author anavarro
 */
public final class SwingMyDoggyConfig
{
    /**
     * Constructor.
     * 
     */
    private SwingMyDoggyConfig()
    {
        // just a Helper class
    }
    
    /**
     * Registers MyDoggy components into the default SwingBuilder
     */
    public static void register()
    {
        register(SwingBuilder.getConfig());
    }
    
    /**
     * Registers MyDoggy components into the specified BuilderConfig
     * 
     * @param config
     *            Builder configuration
     */
    public static void register(BuilderConfig config)
    {
        // visual components
        config.addType(MyDoggyToolWindowManager.class);
        config.addType(MyDoggyToolWindow.class);
        config.addType(MyDoggyContentWindow.class);
        config.addType(MyDoggyToolWindowPerspective.class);
        config.forType(MyDoggyToolWindowManager.class).finishProcessor(MyDoggyToolWindowManagerTypeHandler.getInstance());
        config.forType(MyDoggyToolWindow.class).localize("title").localize("tip");
        config.forType(MyDoggyContent.class).localize("title");
        
        //TODO pull in SwingJavaBuilder
        config.forType(Dimension.class).valueHandler(DimensionAsValueHandler.getInstance());
        
    
    }
    
}
