/**
 * 
 */
package org.javabuilders.swing.mydoggy.handler.type;

import java.awt.Component;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.javabuilders.BuildException;
import org.javabuilders.BuildProcess;
import org.javabuilders.BuilderConfig;
import org.javabuilders.Node;
import org.javabuilders.handler.ITypeHandlerFinishProcessor;
import org.javabuilders.swing.mydoggy.MyDoggyContentWindow;
import org.javabuilders.swing.mydoggy.MyDoggyToolWindow;
import org.javabuilders.swing.mydoggy.MyDoggyToolWindowPerspective;
import org.noos.xing.mydoggy.DockedTypeDescriptor;
import org.noos.xing.mydoggy.FloatingLiveTypeDescriptor;
import org.noos.xing.mydoggy.FloatingTypeDescriptor;
import org.noos.xing.mydoggy.SlidingTypeDescriptor;
import org.noos.xing.mydoggy.TabbedContentManagerUI;
import org.noos.xing.mydoggy.TabbedContentUI;
import org.noos.xing.mydoggy.ToolWindow;
import org.noos.xing.mydoggy.ToolWindowAnchor;
import org.noos.xing.mydoggy.ToolWindowGroup;
import org.noos.xing.mydoggy.ToolWindowManager;
import org.noos.xing.mydoggy.ToolWindowType;
import org.noos.xing.mydoggy.TabbedContentManagerUI.TabLayout;
import org.noos.xing.mydoggy.TabbedContentManagerUI.TabPlacement;
import org.noos.xing.mydoggy.plaf.MyDoggyToolWindowManager;

/**
 * Handles MyDoggyToolWindowManager
 * 
 * @author anavarro
 * 
 */
public final class MyDoggyToolWindowManagerTypeHandler
    implements ITypeHandlerFinishProcessor
{
    
    /**
     * Logger for this class
     */
    private static final Log                                 LOGGER    = LogFactory.getLog(MyDoggyToolWindowManagerTypeHandler.class);
    
    /**
     * singleton
     */
    private static final MyDoggyToolWindowManagerTypeHandler singleton = new MyDoggyToolWindowManagerTypeHandler();
    
    /**
     * @return Singleton
     */
    public static MyDoggyToolWindowManagerTypeHandler getInstance()
    {
        return singleton;
    }
    
    /**
     * (non-Javadoc)
     * 
     * @see org.javabuilders.handler.ITypeHandlerFinishProcessor#finish(org.javabuilders.BuilderConfig, org.javabuilders.BuildProcess,
     *      org.javabuilders.Node, java.lang.String, java.util.Map)
     */
    @SuppressWarnings( { "boxing" })
    public void finish(BuilderConfig config, BuildProcess process, Node current, String key, Map<String, Object> typeDefinition)
        throws BuildException
    {
        final ToolWindowManager toolWindowManager = (MyDoggyToolWindowManager) current.getMainObject();
        final Set<Node> toolWindowPerspectives = current.getContentNodes(MyDoggyToolWindowPerspective.class);
        if (LOGGER.isDebugEnabled())
        {
            LOGGER.debug("toolWindowPerspectives=" + toolWindowPerspectives);
        }
        for (final Node node : toolWindowPerspectives)
        {
            final MyDoggyToolWindowPerspective myDoggyToolWindowPerspective = (MyDoggyToolWindowPerspective) node.getMainObject();
            if (LOGGER.isDebugEnabled())
            {
                LOGGER.debug("myDoggyToolWindowPerspective=" + myDoggyToolWindowPerspective);
            }
            if (myDoggyToolWindowPerspective.getName() != null)
            {
                final ToolWindowGroup toolWindowGroup = toolWindowManager.getToolWindowGroup(myDoggyToolWindowPerspective.getName());
                if (myDoggyToolWindowPerspective.isVisible() != null)
                {
                    toolWindowGroup.setVisible(myDoggyToolWindowPerspective.isVisible());
                }

                addWindows(node, toolWindowManager, toolWindowGroup);
            }
            else
            {
                LOGGER.warn("You must set name for a myDoggyToolWindowPerspective, this toolWindowGroup will be not added.");
            }
        }
        addWindows(current, toolWindowManager, null);
    }
    
    /**
     * addWindows.
     * 
     * @param current
     * @param toolWindowManager
     * @param toolWindowGroup
     */
    @SuppressWarnings({ "boxing", "unchecked" })
    private void addWindows(final Node current, final ToolWindowManager toolWindowManager, final ToolWindowGroup toolWindowGroup)
    {
        final Set<Node> toolWindows = current.getContentNodes(MyDoggyToolWindow.class);
        if (LOGGER.isDebugEnabled())
        {
            LOGGER.debug("toolWindows=" + toolWindows);
        }
        for (final Node node : toolWindows)
        {
            final MyDoggyToolWindow toolWindow = (MyDoggyToolWindow) node.getMainObject();
            if (LOGGER.isDebugEnabled())
            {
                LOGGER.debug("toolWindow=" + toolWindow);
            }
            final Set<Node> componentInsideToolWindows = node.getChildNodes(Component.class);
            if (toolWindow.getId() != null && toolWindow.getTitle() != null && toolWindow.getAnchor() != null
                    && (componentInsideToolWindows != null && componentInsideToolWindows.size() > 0))
            {
                toolWindowManager.registerToolWindow(toolWindow.getId(), toolWindow.getTitle(), toolWindow.getIcon(),
                                                     (Component) componentInsideToolWindows.iterator().next().getMainObject(),
                                                     ToolWindowAnchor.valueOf(toolWindow.getAnchor().toUpperCase()));
                
                final ToolWindow registeredToolWindow = toolWindowManager.getToolWindow(toolWindow.getId());
                if (toolWindow.getActive() != null)
                {
                    registeredToolWindow.setActive(toolWindow.getActive());
                }
                if (toolWindow.getAggregateMode() != null)
                {
                    registeredToolWindow.setAggregateMode(toolWindow.getAggregateMode());
                }
                if (toolWindow.getAutohide() != null)
                {
                    registeredToolWindow.setAutoHide(toolWindow.getAutohide());
                }
                if (toolWindow.getAvailable() != null)
                {
                    registeredToolWindow.setAvailable(toolWindow.getAvailable());
                }
                if (toolWindow.getFlashing() != null)
                {
                    registeredToolWindow.setFlashing(toolWindow.getFlashing());
                }
                if (toolWindow.getMaximized() != null)
                {
                    registeredToolWindow.setMaximized(toolWindow.getMaximized());
                }
                if (toolWindow.getRepresentativeAnchorButtonVisible() != null)
                {
                    registeredToolWindow.setRepresentativeAnchorButtonVisible(toolWindow.getRepresentativeAnchorButtonVisible());
                }
                if (toolWindow.getSelected() != null)
                {
                    registeredToolWindow.setSelected(toolWindow.getSelected());
                }
                if (toolWindow.getVisible() != null)
                {
                    registeredToolWindow.setVisible(toolWindow.getVisible());
                }
                
                // Docked
                final DockedTypeDescriptor dockedTypeDescriptor = (DockedTypeDescriptor) registeredToolWindow
                        .getTypeDescriptor(ToolWindowType.DOCKED);
                if (toolWindow.getDockLength() != null)
                {
                    dockedTypeDescriptor.setDockLength(toolWindow.getDockLength().intValue());
                }
                if (toolWindow.getDockHideRepresentativeButtonOnVisible() != null)
                {
                    dockedTypeDescriptor.setHideRepresentativeButtonOnVisible(toolWindow.getDockHideRepresentativeButtonOnVisible());
                }
                if (toolWindow.getMinimumDockLength() != null)
                {
                    dockedTypeDescriptor.setMinimumDockLength(toolWindow.getMinimumDockLength().intValue());
                }
                if (toolWindow.getDockPopupMenuEnabled() != null)
                {
                    dockedTypeDescriptor.setPopupMenuEnabled(toolWindow.getDockPopupMenuEnabled());
                }
                if (toolWindow.getDockPreviewDelay() != null)
                {
                    dockedTypeDescriptor.setPreviewDelay(toolWindow.getDockPreviewDelay().intValue());
                }
                if (toolWindow.getDockPreviewEnabled() != null)
                {
                    dockedTypeDescriptor.setPreviewEnabled(toolWindow.getDockPreviewEnabled());
                }
                if (toolWindow.getDockPreviewTransparentRatio() != null)
                {
                    dockedTypeDescriptor.setPreviewTransparentRatio(toolWindow.getDockPreviewTransparentRatio().floatValue());
                }
                
                // Sliding
                final SlidingTypeDescriptor slidingTypeDescriptor = (SlidingTypeDescriptor) registeredToolWindow
                        .getTypeDescriptor(ToolWindowType.SLIDING);
                if (toolWindow.getSlidingTransparentDelay() != null)
                {
                    slidingTypeDescriptor.setTransparentDelay(toolWindow.getSlidingTransparentDelay().intValue());
                }
                if (toolWindow.getSlidingTransparentMode() != null)
                {
                    slidingTypeDescriptor.setTransparentMode(toolWindow.getSlidingTransparentMode());
                }
                if (toolWindow.getSlidingTransparentRatio() != null)
                {
                    slidingTypeDescriptor.setTransparentRatio(toolWindow.getSlidingTransparentRatio().floatValue());
                }
                
                // Floating
                final FloatingTypeDescriptor floatingTypeDescriptor = (FloatingTypeDescriptor) registeredToolWindow
                        .getTypeDescriptor(ToolWindowType.FLOATING);
                if (toolWindow.getFloatingAddToTaskBar() != null)
                {
                    floatingTypeDescriptor.setAddToTaskBar(toolWindow.getFloatingAddToTaskBar());
                }
                if (toolWindow.getFloatingModal() != null)
                {
                    floatingTypeDescriptor.setModal(toolWindow.getFloatingModal());
                }
                if (toolWindow.getFloatingSize() != null)
                {
                    floatingTypeDescriptor.setSize((int) toolWindow.getFloatingSize().getWidth(), (int) toolWindow.getFloatingSize().getHeight());
                }
                if (toolWindow.getFloatingTransparentDelay() != null)
                {
                    floatingTypeDescriptor.setTransparentDelay(toolWindow.getFloatingTransparentDelay().intValue());
                }
                if (toolWindow.getFloatingTransparentMode() != null)
                {
                    floatingTypeDescriptor.setTransparentMode(toolWindow.getFloatingTransparentMode());
                }
                if (toolWindow.getFloatingTransparentRatio() != null)
                {
                    floatingTypeDescriptor.setTransparentRatio(toolWindow.getFloatingTransparentRatio().floatValue());
                }
                
                // Floating Free
                final FloatingTypeDescriptor floatingFreeTypeDescriptor = (FloatingTypeDescriptor) registeredToolWindow
                        .getTypeDescriptor(ToolWindowType.FLOATING_FREE);
                
                if (toolWindow.getFloatingFreeAddToTaskBar() != null)
                {
                    floatingFreeTypeDescriptor.setAddToTaskBar(toolWindow.getFloatingFreeAddToTaskBar());
                }
                if (toolWindow.getFloatingFreeModal() != null)
                {
                    floatingFreeTypeDescriptor.setModal(toolWindow.getFloatingFreeModal());
                }
                if (toolWindow.getFloatingFreeSize() != null)
                {
                    floatingFreeTypeDescriptor.setSize((int) toolWindow.getFloatingFreeSize().getWidth(), (int) toolWindow.getFloatingFreeSize().getHeight());
                }
                if (toolWindow.getFloatingFreeTransparentDelay() != null)
                {
                    floatingFreeTypeDescriptor.setTransparentDelay(toolWindow.getFloatingFreeTransparentDelay().intValue());
                }
                if (toolWindow.getFloatingFreeTransparentMode() != null)
                {
                    floatingFreeTypeDescriptor.setTransparentMode(toolWindow.getFloatingFreeTransparentMode());
                }
                if (toolWindow.getFloatingFreeTransparentRatio() != null)
                {
                    floatingFreeTypeDescriptor.setTransparentRatio(toolWindow.getFloatingFreeTransparentRatio().floatValue());
                }
                
                // Floating Live
                final FloatingLiveTypeDescriptor floatingLiveTypeDescriptor = (FloatingLiveTypeDescriptor) registeredToolWindow
                        .getTypeDescriptor(ToolWindowType.FLOATING_LIVE);
                if (toolWindow.getFloatingLiveSize() != null)
                {
                    floatingLiveTypeDescriptor.setSize((int) toolWindow.getFloatingLiveSize().getWidth(), (int) toolWindow.getFloatingLiveSize().getHeight());
                }
                if (toolWindow.getFloatingLiveTransparentDelay() != null)
                {
                    floatingLiveTypeDescriptor.setTransparentDelay(toolWindow.getFloatingLiveTransparentDelay().intValue());
                }
                if (toolWindow.getFloatingLiveTransparentMode() != null)
                {
                    floatingLiveTypeDescriptor.setTransparentMode(toolWindow.getFloatingLiveTransparentMode());
                }
                if (toolWindow.getFloatingLiveTransparentRatio() != null)
                {
                    floatingLiveTypeDescriptor.setTransparentRatio(toolWindow.getFloatingLiveTransparentRatio().floatValue());
                }
                
                // Add to the toolWindowGroup if it exists
                if (toolWindowGroup != null)
                {
                    toolWindowGroup.addToolWindow(registeredToolWindow);
                }
            }
            else
            {
                LOGGER.warn("You must set the id=" + toolWindow.getId() + ", the title=" + toolWindow.getTitle() + ", the anchor="
                        + toolWindow.getAnchor() + " and add a Component for MyDoggyToolWindow a component inside="
                        + componentInsideToolWindows);
            }
        }
        
        final Set<Node> contentWindows = current.getContentNodes(MyDoggyContentWindow.class);
        for (final Node node : contentWindows)
        {
            final MyDoggyContentWindow contentWindow = (MyDoggyContentWindow) node.getMainObject();
            if (LOGGER.isDebugEnabled())
            {
                LOGGER.debug("contentWindow=" + contentWindow);
            }
            final Set<Node> componentInsideContentWindows = node.getChildNodes(Component.class);
            if (contentWindow.getId() != null && contentWindow.getTitle() != null
                    && (componentInsideContentWindows != null && componentInsideContentWindows.size() > 0))
            {
                toolWindowManager.getContentManager().addContent(contentWindow.getId(),
                                                                 contentWindow.getTitle(),
                                                                 contentWindow.getIcon(),
                                                                 (Component) componentInsideContentWindows.iterator().next()
                                                                         .getMainObject());
                final TabbedContentManagerUI<TabbedContentUI> contentManagerUI = (TabbedContentManagerUI) toolWindowManager
                        .getContentManager().getContentManagerUI();
                contentManagerUI.setShowAlwaysTab(true);
                if (contentWindow.getCloseable() != null)
                {
                    contentManagerUI.setCloseable(contentWindow.getCloseable());
                }
                if (contentWindow.getDetachable() != null)
                {
                    contentManagerUI.setCloseable(contentWindow.getDetachable());
                }
                if (contentWindow.getMinimizable() != null)
                {
                    contentManagerUI.setCloseable(contentWindow.getMinimizable());
                }
                if (contentWindow.getShowAlwaysTab() != null)
                {
                    contentManagerUI.setCloseable(contentWindow.getShowAlwaysTab());
                }
                if (contentWindow.getTabLayout() != null)
                {
                    contentManagerUI.setTabLayout(TabLayout.valueOf(contentWindow.getTabLayout().toUpperCase()));
                }
                if (contentWindow.getTabPlacement() != null)
                {
                    contentManagerUI.setTabPlacement(TabPlacement.valueOf(contentWindow.getTabPlacement().toUpperCase()));
                }
            }
            else
            {
                LOGGER.warn("You must set the id, the title and add a Component for MyDoggyContentWindow");
            }
        }
    }
    
}
